@@ -1,10 +1,11 @@
 /*
     __ _____ _____ _____
  __|  |   __|     |   | |  JSON for Modern C++ (test suite)
-|  |  |__   |  |  | | | |  version 3.1.2
+|  |  |__   |  |  | | | |  version 3.5.0
 |_____|_____|_____|_|___|  https://github.com/nlohmann/json
 
 Licensed under the MIT License <http://opensource.org/licenses/MIT>.
+SPDX-License-Identifier: MIT
 Copyright (c) 2013-2018 Niels Lohmann <http://nlohmann.me>.
 
 Permission is hereby  granted, free of charge, to any  person obtaining a copy
@@ -31,13 +32,20 @@ SOFTWARE.
 #include <nlohmann/json.hpp>
 using nlohmann::json;
 
+#if (defined(__cplusplus) && __cplusplus >= 201703L) || (defined(_HAS_CXX17) && _HAS_CXX17 == 1) // fix for issue #464
+    #define JSON_HAS_CPP_17
+    #define JSON_HAS_CPP_14
+#elif (defined(__cplusplus) && __cplusplus >= 201402L) || (defined(_HAS_CXX14) && _HAS_CXX14 == 1)
+    #define JSON_HAS_CPP_14
+#endif
+
 TEST_CASE("iterator_wrapper")
 {
     SECTION("object")
     {
         SECTION("value")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto i : json::iterator_wrapper(j))
@@ -70,7 +78,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("reference")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto& i : json::iterator_wrapper(j))
@@ -109,12 +117,12 @@ TEST_CASE("iterator_wrapper")
             CHECK(counter == 3);
 
             // check if values where changed
-            CHECK(j == json({{"A", 11}, {"B", 22}}));
+            CHECK(j == json({ {"A", 11}, {"B", 22} }));
         }
 
         SECTION("const value")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto i : json::iterator_wrapper(j))
@@ -147,7 +155,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("const reference")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto& i : json::iterator_wrapper(j))
@@ -183,7 +191,7 @@ TEST_CASE("iterator_wrapper")
     {
         SECTION("value")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto i : json::iterator_wrapper(j))
@@ -216,7 +224,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("reference")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto& i : json::iterator_wrapper(j))
@@ -249,7 +257,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("const value")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto i : json::iterator_wrapper(j))
@@ -282,7 +290,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("const reference")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto& i : json::iterator_wrapper(j))
@@ -318,7 +326,7 @@ TEST_CASE("iterator_wrapper")
     {
         SECTION("value")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (auto i : json::iterator_wrapper(j))
@@ -351,7 +359,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("reference")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (auto& i : json::iterator_wrapper(j))
@@ -390,12 +398,12 @@ TEST_CASE("iterator_wrapper")
             CHECK(counter == 3);
 
             // check if values where changed
-            CHECK(j == json({"AA", "BB"}));
+            CHECK(j == json({ "AA", "BB" }));
         }
 
         SECTION("const value")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (const auto i : json::iterator_wrapper(j))
@@ -428,7 +436,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("const reference")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (const auto& i : json::iterator_wrapper(j))
@@ -464,7 +472,7 @@ TEST_CASE("iterator_wrapper")
     {
         SECTION("value")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (auto i : json::iterator_wrapper(j))
@@ -497,7 +505,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("reference")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (auto& i : json::iterator_wrapper(j))
@@ -530,7 +538,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("const value")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (const auto i : json::iterator_wrapper(j))
@@ -563,7 +571,7 @@ TEST_CASE("iterator_wrapper")
 
         SECTION("const reference")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (const auto& i : json::iterator_wrapper(j))
@@ -734,7 +742,7 @@ TEST_CASE("items()")
     {
         SECTION("value")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto i : j.items())
@@ -767,7 +775,7 @@ TEST_CASE("items()")
 
         SECTION("reference")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto& i : j.items())
@@ -806,12 +814,12 @@ TEST_CASE("items()")
             CHECK(counter == 3);
 
             // check if values where changed
-            CHECK(j == json({{"A", 11}, {"B", 22}}));
+            CHECK(j == json({ {"A", 11}, {"B", 22} }));
         }
 
         SECTION("const value")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto i : j.items())
@@ -844,7 +852,7 @@ TEST_CASE("items()")
 
         SECTION("const reference")
         {
-            json j = {{"A", 1}, {"B", 2}};
+            json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto& i : j.items())
@@ -874,13 +882,29 @@ TEST_CASE("items()")
 
             CHECK(counter == 3);
         }
+
+#ifdef JSON_HAS_CPP_17
+        SECTION("structured bindings")
+        {
+            json j = { {"A", 1}, {"B", 2} };
+
+            std::map<std::string, int> m;
+
+            for (auto const&[key, value] : j.items())
+            {
+                m.emplace(key, value);
+            }
+
+            CHECK(j.get<decltype(m)>() == m);
+        }
+#endif
     }
 
     SECTION("const object")
     {
         SECTION("value")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto i : j.items())
@@ -913,7 +937,7 @@ TEST_CASE("items()")
 
         SECTION("reference")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (auto& i : j.items())
@@ -946,7 +970,7 @@ TEST_CASE("items()")
 
         SECTION("const value")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto i : j.items())
@@ -979,7 +1003,7 @@ TEST_CASE("items()")
 
         SECTION("const reference")
         {
-            const json j = {{"A", 1}, {"B", 2}};
+            const json j = { {"A", 1}, {"B", 2} };
             int counter = 1;
 
             for (const auto& i : j.items())
@@ -1015,7 +1039,7 @@ TEST_CASE("items()")
     {
         SECTION("value")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (auto i : j.items())
@@ -1048,7 +1072,7 @@ TEST_CASE("items()")
 
         SECTION("reference")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (auto& i : j.items())
@@ -1087,12 +1111,12 @@ TEST_CASE("items()")
             CHECK(counter == 3);
 
             // check if values where changed
-            CHECK(j == json({"AA", "BB"}));
+            CHECK(j == json({ "AA", "BB" }));
         }
 
         SECTION("const value")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (const auto i : j.items())
@@ -1125,7 +1149,7 @@ TEST_CASE("items()")
 
         SECTION("const reference")
         {
-            json j = {"A", "B"};
+            json j = { "A", "B" };
             int counter = 1;
 
             for (const auto& i : j.items())
@@ -1161,7 +1185,7 @@ TEST_CASE("items()")
     {
         SECTION("value")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (auto i : j.items())
@@ -1194,7 +1218,7 @@ TEST_CASE("items()")
 
         SECTION("reference")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (auto& i : j.items())
@@ -1227,7 +1251,7 @@ TEST_CASE("items()")
 
         SECTION("const value")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (const auto i : j.items())
@@ -1260,7 +1284,7 @@ TEST_CASE("items()")
 
         SECTION("const reference")
         {
-            const json j = {"A", "B"};
+            const json j = { "A", "B" };
             int counter = 1;
 
             for (const auto& i : j.items())
