@@ -1,11 +1,12 @@
 /*
     __ _____ _____ _____
  __|  |   __|     |   | |  JSON for Modern C++ (test suite)
-|  |  |__   |  |  | | | |  version 2.0.3
+|  |  |__   |  |  | | | |  version 3.5.0
 |_____|_____|_____|_|___|  https://github.com/nlohmann/json
 
 Licensed under the MIT License <http://opensource.org/licenses/MIT>.
-Copyright (c) 2013-2016 Niels Lohmann <http://nlohmann.me>.
+SPDX-License-Identifier: MIT
+Copyright (c) 2013-2018 Niels Lohmann <http://nlohmann.me>.
 
 Permission is hereby  granted, free of charge, to any  person obtaining a copy
 of this software and associated  documentation files (the "Software"), to deal
@@ -29,7 +30,7 @@ SOFTWARE.
 #include "catch.hpp"
 
 #define private public
-#include "json.hpp"
+#include <nlohmann/json.hpp>
 using nlohmann::json;
 
 TEST_CASE("iterators 1")
@@ -225,27 +226,135 @@ TEST_CASE("iterators 1")
                 CHECK(*it == j_const);
             }
 
+            SECTION("additional tests")
+            {
+                SECTION("!(begin != begin)")
+                {
+                    CHECK(not(j.begin() != j.begin()));
+                }
+
+                SECTION("!(end != end)")
+                {
+                    CHECK(not(j.end() != j.end()));
+                }
+
+                SECTION("begin < end")
+                {
+                    CHECK(j.begin() < j.end());
+                }
+
+                SECTION("begin <= end")
+                {
+                    CHECK(j.begin() <= j.end());
+                }
+
+                SECTION("end > begin")
+                {
+                    CHECK(j.end() > j.begin());
+                }
+
+                SECTION("end >= begin")
+                {
+                    CHECK(j.end() >= j.begin());
+                }
+
+                SECTION("end == end")
+                {
+                    CHECK(j.end() == j.end());
+                }
+
+                SECTION("end <= end")
+                {
+                    CHECK(j.end() <= j.end());
+                }
+
+                SECTION("begin == begin")
+                {
+                    CHECK(j.begin() == j.begin());
+                }
+
+                SECTION("begin <= begin")
+                {
+                    CHECK(j.begin() <= j.begin());
+                }
+
+                SECTION("begin >= begin")
+                {
+                    CHECK(j.begin() >= j.begin());
+                }
+
+                SECTION("!(begin == end)")
+                {
+                    CHECK(not(j.begin() == j.end()));
+                }
+
+                SECTION("begin != end")
+                {
+                    CHECK(j.begin() != j.end());
+                }
+
+                SECTION("begin+1 == end")
+                {
+                    CHECK(j.begin() + 1 == j.end());
+                }
+
+                SECTION("begin == end-1")
+                {
+                    CHECK(j.begin() == j.end() - 1);
+                }
+
+                SECTION("begin != end+1")
+                {
+                    CHECK(j.begin() != j.end() + 1);
+                }
+
+                SECTION("end != end+1")
+                {
+                    CHECK(j.end() != j.end() + 1);
+                }
+
+                SECTION("begin+1 != begin+2")
+                {
+                    CHECK(j.begin() + 1 != j.begin() + 2);
+                }
+
+                SECTION("begin+1 < begin+2")
+                {
+                    CHECK(j.begin() + 1 < j.begin() + 2);
+                }
+
+                SECTION("begin+1 <= begin+2")
+                {
+                    CHECK(j.begin() + 1 <= j.begin() + 2);
+                }
+
+                SECTION("end+1 != end+2")
+                {
+                    CHECK(j.end() + 1 != j.end() + 2);
+                }
+            }
+
             SECTION("key/value")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(it.value() == json(true));
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(cit.value() == json(true));
 
                 auto rit = j.rend();
                 auto crit = j.crend();
-                CHECK_THROWS_AS(rit.key(), std::domain_error);
-                CHECK_THROWS_AS(rit.value(), std::out_of_range);
-                CHECK_THROWS_AS(crit.key(), std::domain_error);
-                CHECK_THROWS_AS(crit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(rit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(rit.value(), "cannot get value");
-                CHECK_THROWS_WITH(crit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(crit.value(), "cannot get value");
+                CHECK_THROWS_AS(rit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(rit.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(rit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(rit.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(crit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(crit.value(), "[json.exception.invalid_iterator.214] cannot get value");
             }
         }
 
@@ -433,23 +542,23 @@ TEST_CASE("iterators 1")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(it.value() == json("hello world"));
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(cit.value() == json("hello world"));
 
                 auto rit = j.rend();
                 auto crit = j.crend();
-                CHECK_THROWS_AS(rit.key(), std::domain_error);
-                CHECK_THROWS_AS(rit.value(), std::out_of_range);
-                CHECK_THROWS_AS(crit.key(), std::domain_error);
-                CHECK_THROWS_AS(crit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(rit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(rit.value(), "cannot get value");
-                CHECK_THROWS_WITH(crit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(crit.value(), "cannot get value");
+                CHECK_THROWS_AS(rit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(rit.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(rit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(rit.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(crit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(crit.value(), "[json.exception.invalid_iterator.214] cannot get value");
             }
         }
 
@@ -630,11 +739,11 @@ TEST_CASE("iterators 1")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(it.value() == json(1));
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(cit.value() == json(1));
             }
         }
@@ -1007,23 +1116,23 @@ TEST_CASE("iterators 1")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(it.value() == json(23));
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(cit.value() == json(23));
 
                 auto rit = j.rend();
                 auto crit = j.crend();
-                CHECK_THROWS_AS(rit.key(), std::domain_error);
-                CHECK_THROWS_AS(rit.value(), std::out_of_range);
-                CHECK_THROWS_AS(crit.key(), std::domain_error);
-                CHECK_THROWS_AS(crit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(rit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(rit.value(), "cannot get value");
-                CHECK_THROWS_WITH(crit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(crit.value(), "cannot get value");
+                CHECK_THROWS_AS(rit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(rit.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(rit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(rit.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(crit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(crit.value(), "[json.exception.invalid_iterator.214] cannot get value");
             }
         }
 
@@ -1211,23 +1320,23 @@ TEST_CASE("iterators 1")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(it.value() == json(23));
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(cit.value() == json(23));
 
                 auto rit = j.rend();
                 auto crit = j.crend();
-                CHECK_THROWS_AS(rit.key(), std::domain_error);
-                CHECK_THROWS_AS(rit.value(), std::out_of_range);
-                CHECK_THROWS_AS(crit.key(), std::domain_error);
-                CHECK_THROWS_AS(crit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(rit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(rit.value(), "cannot get value");
-                CHECK_THROWS_WITH(crit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(crit.value(), "cannot get value");
+                CHECK_THROWS_AS(rit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(rit.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(rit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(rit.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(crit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(crit.value(), "[json.exception.invalid_iterator.214] cannot get value");
             }
         }
 
@@ -1415,23 +1524,23 @@ TEST_CASE("iterators 1")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(it.value() == json(23.42));
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
                 CHECK(cit.value() == json(23.42));
 
                 auto rit = j.rend();
                 auto crit = j.crend();
-                CHECK_THROWS_AS(rit.key(), std::domain_error);
-                CHECK_THROWS_AS(rit.value(), std::out_of_range);
-                CHECK_THROWS_AS(crit.key(), std::domain_error);
-                CHECK_THROWS_AS(crit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(rit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(rit.value(), "cannot get value");
-                CHECK_THROWS_WITH(crit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(crit.value(), "cannot get value");
+                CHECK_THROWS_AS(rit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(rit.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(rit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(rit.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(crit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(crit.value(), "[json.exception.invalid_iterator.214] cannot get value");
             }
         }
 
@@ -1489,26 +1598,94 @@ TEST_CASE("iterators 1")
             {
                 auto it = j.begin();
                 auto cit = j_const.cbegin();
-                CHECK_THROWS_AS(it.key(), std::domain_error);
-                CHECK_THROWS_AS(it.value(), std::out_of_range);
-                CHECK_THROWS_AS(cit.key(), std::domain_error);
-                CHECK_THROWS_AS(cit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(it.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(it.value(), "cannot get value");
-                CHECK_THROWS_WITH(cit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(cit.value(), "cannot get value");
+                CHECK_THROWS_AS(it.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(it.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(cit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(cit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(it.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(it.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(cit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(cit.value(), "[json.exception.invalid_iterator.214] cannot get value");
 
                 auto rit = j.rend();
                 auto crit = j.crend();
-                CHECK_THROWS_AS(rit.key(), std::domain_error);
-                CHECK_THROWS_AS(rit.value(), std::out_of_range);
-                CHECK_THROWS_AS(crit.key(), std::domain_error);
-                CHECK_THROWS_AS(crit.value(), std::out_of_range);
-                CHECK_THROWS_WITH(rit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(rit.value(), "cannot get value");
-                CHECK_THROWS_WITH(crit.key(), "cannot use key() for non-object iterators");
-                CHECK_THROWS_WITH(crit.value(), "cannot get value");
+                CHECK_THROWS_AS(rit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(rit.value(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.key(), json::invalid_iterator&);
+                CHECK_THROWS_AS(crit.value(), json::invalid_iterator&);
+                CHECK_THROWS_WITH(rit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(rit.value(), "[json.exception.invalid_iterator.214] cannot get value");
+                CHECK_THROWS_WITH(crit.key(), "[json.exception.invalid_iterator.207] cannot use key() for non-object iterators");
+                CHECK_THROWS_WITH(crit.value(), "[json.exception.invalid_iterator.214] cannot get value");
             }
         }
     }
+
+    SECTION("conversion from iterator to const iterator")
+    {
+        SECTION("boolean")
+        {
+            json j = true;
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("string")
+        {
+            json j = "hello world";
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("array")
+        {
+            json j = {1, 2, 3};
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("object")
+        {
+            json j = {{"A", 1}, {"B", 2}, {"C", 3}};
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("number (integer)")
+        {
+            json j = 23;
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("number (unsigned)")
+        {
+            json j = 23u;
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("number (float)")
+        {
+            json j = 23.42;
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+        SECTION("null")
+        {
+            json j = nullptr;
+            json::const_iterator it = j.begin();
+            CHECK(it == j.cbegin());
+            it = j.begin();
+            CHECK(it == j.cbegin());
+        }
+    }
 }
